package net.sergeych.mptools

import net.sergeych.ugdx.shared.api.unit.UnitApplication
import kotlinx.datetime.Instant
import kotlinx.serialization.Serializable
import net.sergeych.boss_serialization_mp.BossEncoder
import net.sergeych.boss_serialization_mp.decodeBoss
import net.sergeych.unikrypto.PrivateKey
import net.sergeych.unikrypto.SignedRecord

@Serializable
class MultisignedDocument<Author>(
    val packedDoc: ByteArray,
    val packedSignatures: List<ByteArray>,
) {
    @Serializable
    private class SignaturePayload<Author>(
        val author: Author,
        val signedAt: Instant = Now()
    )

    inline fun <reified T>getDocument(): T = packedDoc.decodeBoss()

    inline fun <reified T>setDocument(newDocument: T): MultisignedDocument<Author> =
        MultisignedDocument(BossEncoder.encode(newDocument), listOf())

    class Signature<Author>(val packed: ByteArray,_signedRecord: SignedRecord? = null) {
        val signedRecord = _signedRecord ?: SignedRecord.unpack(packed)
        private val payload: SignaturePayload<Author> = signedRecord.decode()
        val author by payload::author
        val signedAt by payload::signedAt
    }

    val signatures: Map<Author,Signature<Author>> by lazy {
        packedSignatures.map { Signature<Author>(it) }.map { it.author to it}.toMap()
    }

    fun isSignedBy(author: Author) = author in signatures

    fun signedAt(author: Author): Instant? = signatures[author]?.signedAt

    fun addSignature(author: UnitApplication.Author, key: PrivateKey): MultisignedDocument<Author> {
        val ss = signatures + (author to Signature(SignedRecord.pack(key, SignaturePayload(author))))
        return MultisignedDocument(packedDoc, ss.values.map { it.packed })
    }
}