@file:Suppress("IMPLICIT_CAST_TO_ANY", "UNCHECKED_CAST")
@file:OptIn(ExperimentalSerializationApi::class)

package net.sergeych.mptools

import kotlinx.datetime.Instant
import kotlinx.serialization.ExperimentalSerializationApi
import net.sergeych.boss_serialization.BossDecoder
import net.sergeych.boss_serialization_mp.BossEncoder
import net.sergeych.bossk.Bossk
import kotlin.reflect.KType
import kotlin.reflect.typeOf

inline fun <reified T> smartEncodeBoss(value: T): ByteArray =
    when (value) {
        is String, is Number, is Boolean, is ByteArray, is Instant -> Bossk.pack(value)
        else -> BossEncoder.encode(value)
    }


inline fun <reified T: Any> ByteArray.smartDecodeBoss(): T {
    val value = Bossk.unpack<Any>(this)
    return when(value) {
        is String, is Number, is Boolean, is ByteArray, is Instant -> value as T
        is Map<*, *> -> BossDecoder.decodeFrom(typeOf<T>(), value as Map<String, Any?>)
        else -> throw IllegalArgumentException("can't unpack, unknown type")
    }
}