@file:OptIn(ExperimentalSerializationApi::class)

package net.sergeych.parsec

import net.sergeych.ugdx.shared.ApiException
import kotlinx.serialization.ExperimentalSerializationApi
import kotlinx.serialization.Serializable
import net.sergeych.boss_serialization_mp.BossEncoder
import net.sergeych.boss_serialization_mp.BossStruct
import net.sergeych.boss_serialization_mp.decodeBoss

@Serializable
internal data class Command(
    val cmd: String,
    val args: BossStruct
) {
    companion object {
        /**
         * Packs parsec binary command (bossed {cmd: string,args: Params})
         */
        fun pack(command: String, args: BossStruct): ByteArray {
            return BossEncoder.encode(Command(command, args))
        }

        /**
         * Unpack parsec command result (bossed map encoded as Params)
         */
        inline fun <reified T>unpackResult(packed: ByteArray): T {
            val payload = packed.decodeBoss<ParsecResult<T>>()
            payload.error?.let {
                // this will throw API Exception if it is known
                ApiException.throwIfKnown(it.code, it.text)
                // and this will throw common error if it is not
                throw ParsecRemoteException(it.code, it.text)
            }
            if( payload.result == null ) throw ParsecException(
                "internal_error",
                "failed to unpack parsec result: ${packed.decodeBoss<BossStruct>()}"
            )
            return payload.result
        }
    }
}

@Serializable
internal data class ParsecError(val code: String, val text: String)

@Serializable
internal data class ParsecResult<T>(
    val result: T? = null,
    val error: ParsecError? = null
)
