package net.sergeych.parsec

import kotlinx.serialization.Serializable
import net.sergeych.boss_serialization_mp.BossStruct
import net.sergeych.boss_serialization_mp.decodeBoss
import net.sergeych.bossk.Bossk
import net.sergeych.mp_logger.*
import net.sergeych.unikrypto.SymmetricKey
import net.sergeych.unikrypto.SymmetricKeys


@Serializable
data class SessionParams(
    val authToken: String,
    val params: ByteArray
)

@Serializable
data class SessionResult(val result: ByteArray)

/**
 * Encrypted connection over a regular connection. It works like this: RootConnection
 * goes first, negotiate params for endpoont, than establish Endpoint over it.
 *
 * @param connection underlying connection that will carry the Endpoint traffic
 * @param sessionKey packed session key for the endpoint
 * @param authToken  string authentication token to identify the Endpoint traffic
 */
class Endpoint(
    private val connection: ParsecConnection,
    sessionKey: ByteArray,
    private val authToken: String
) : ParsecConnection, Loggable by LogTag("PRSEP") {

    val key = SymmetricKeys.create(sessionKey)

    override suspend fun call(command: String, params: BossStruct): BossStruct {
        info { ">>:: $command: $params" }
        val data: SessionResult = connection.exec("cmd", SessionParams(
            authToken,
            key.etaEncrypt(Command.pack(command, params))
        ))
        info { "<<:: $command ${Bossk.unpack<BossStruct>(key.etaDecrypt(data.result))}" }
        return Command.unpackResult(key.etaDecrypt(data.result))
    }

}