package net.sergeych.parsec

import kotlinx.serialization.Serializable
import net.sergeych.mp_tools.getInt
import net.sergeych.mp_tools.putInt
import net.sergeych.unikrypto.HashAlgorithm
import net.sergeych.unikrypto.digest

@Serializable
data class POWTask(val type: Int, val length: Int, val source: ByteArray) {

    fun solve(): ByteArray {
        if (type != 1) throw IllegalArgumentException("POW type $type is not supported")
        return solvePOW1(source, length)
    }

    fun check(solution: ByteArray): Boolean {
        if (type != 1) throw IllegalArgumentException("POW type $type is not supported")
        return checkPOW1(source, solution, length)
    }

    companion object {

        fun countZeroBits(source: ByteArray): Int {
            var mask = 1
            var counter = 0
            var pos = 0
            while (pos < source.size && (source[pos].toInt() and mask) == 0) {
                counter += 1
                mask = mask shl 1
                if (mask > 0x80 && pos < source.size) {
                    mask = 1
                    pos += 1
                }
            }
            return counter
        }

        fun checkPOW1(source: ByteArray, solution: ByteArray, length: Int): Boolean {
            val s = HashAlgorithm.SHA3_384.digest(solution, source);
            return countZeroBits(s) == length
        }

        fun solvePOW1(source: ByteArray, length: Int): ByteArray {
            val buffer = ByteArray(8)
            var index = 0
            while (index < 2) {
                val solution = buffer
                if (checkPOW1(source, solution, length))
                    return solution
                val x = buffer.getInt(index) + 1
                buffer.putInt(index, x)
                if (x == 0x7FFFffff)
                    index++;
            }
            throw Exception("failed to solve POW1 of length $length")
        }

    }
}