package net.sergeych.parsec

import net.sergeych.boss_serialization_mp.BossStruct
import io.ktor.client.*
import io.ktor.client.features.websocket.*
import io.ktor.client.request.forms.*
import io.ktor.client.statement.*
import io.ktor.http.*
import kotlinx.serialization.ExperimentalSerializationApi
import net.sergeych.boss_serialization_mp.BossEncoder
import net.sergeych.boss_serialization_mp.decodeBoss


class RootConnection(val url: String) : ParsecConnection {

    var traceCommands = false

    val client = HttpClient() {
        install(WebSockets) {
        }
    }

    @OptIn(ExperimentalSerializationApi::class)
    override suspend fun call(command: String, params: BossStruct): BossStruct {
        if (traceCommands)
            println("RC:>> $command: $params")

        val response: HttpStatement = client.submitFormWithBinaryData(
            url,
            formData = formData {
                append("rem","this should be ignodred")
                append("cmd", BossEncoder.encode(Command(command, params)), Headers.build {
                    append(HttpHeaders.ContentType, "application/octet-stream")
                    append(HttpHeaders.ContentDisposition, "filename=command.bin")
                }
                )
            }
        )

        val answer = response.receive<ByteArray>().decodeBoss<ParsecResult<BossStruct>>()
        if (traceCommands) println("RC:<< $answer")

        answer.error?.let { throw ParsecException(it.code, it.text) }

        return answer.result ?: BossStruct.EMPTY
    }

}