package net.sergeych.ugdx.shared.api

import net.sergeych.ugdx.shared.ErrorCode
import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable
import net.sergeych.boss_serialization_mp.BossStruct

@Serializable
sealed class AsyncBlock {
    @Serializable
    @SerialName("nack")
    object Nack: AsyncBlock()

    @Serializable
    @SerialName("ack")
    class Ack(val sessionId: String,val encryptedCredentials: ByteArray): AsyncBlock()

    @Serializable
    @SerialName("acpt")
    object Accept: AsyncBlock()

    @Serializable
    @SerialName("data")
    class Data(val encrypted: ByteArray): AsyncBlock()
}

@Serializable
sealed class PayloadBlock {
    @Serializable
    @SerialName("rq")
    class Request(val id: Int,val cmd: String,val args: BossStruct): PayloadBlock()

    @Serializable
    @SerialName("rsp")
    class Response(val toId: Int,val result: ByteArray?=null): PayloadBlock()

    @Serializable
    @SerialName("err")
    class Error(val toId: Int,val code: ErrorCode,val text: String): PayloadBlock()

    @Serializable
    @SerialName("psh")
    class Push(val payload: PushPayload): PayloadBlock()
}


