package net.sergeych.ugdx.shared.api.unit

import kotlinx.datetime.Instant
import net.sergeych.boss_serialization_mp.decodeBoss
import net.sergeych.mp_logger.LogTag
import net.sergeych.mp_logger.Loggable
import net.sergeych.unikrypto.SigningKey
import net.sergeych.unikrypto.VerifyingKey

@kotlinx.serialization.Serializable
class UnitApplicationPack(
    val packed: ByteArray,
    val providedSignatures: List<Signature>,
) : Loggable by LogTag("UAPCK") {

    @kotlinx.serialization.Serializable
    class Signature(
        val author: UnitApplication.Author,
        val key: VerifyingKey,
        val signature: ByteArray,
        val signedAt: Instant
    ) {
        fun check(packed: ByteArray): Boolean {
            return key.checkSignature(packed, signature)
        }
    }

    val application: UnitApplication by lazy { packed.decodeBoss<UnitApplication>() }

    val signatures: Set<UnitApplication.Author> by lazy {
        providedSignatures.filter { it.check(packed) }.map { it.author }.toSet()
    }

    val signedByAgent: Boolean by lazy { UnitApplication.Author.Agent in signatures }
    val signedByApplicant: Boolean by lazy { UnitApplication.Author.Applicant in signatures }

    fun signBy(author: UnitApplication.Author,key: SigningKey) {

    }

}