package net.sergeych.ugdx.shared

import net.sergeych.boss_serialization_mp.KVBinaryStorage
import net.sergeych.unikrypto.EncryptedBinaryStorage
import net.sergeych.unikrypto.SymmetricKey

/**
 * Create a platform-specific permanent binary storage, suitable to use with GuestAPI. Just pass it
 * to [GusetAPI.setup]:
 * ~~~
 * GuestAPI.setup {
 *   storage = PermanentBinaryStorage("mayAppName")
 * }
 * ~~~
 * Note that the storage itself is not encrypted, see [EncryptedPermanentBinaryStorage] for a safe version
 */
expect fun PermanentBinaryStorage(name: String): KVBinaryStorage

/**
 * Combniation of encrypted and platofrm-specific permanent binary storage. As there is no multiplatform source
 * of per-application encrypted keystore, you should provide encryption key.
 */
fun EncryptedPermanentBinaryStorage(name: String,key: SymmetricKey,clearOnOpenError: Boolean): KVBinaryStorage {
    return EncryptedBinaryStorage(net.sergeych.ugdx.shared.PermanentBinaryStorage(name), key,clearOnOpenError)
}

